#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                 -*-  Object Testing node  -*-                                #
#                                                                              #
#                                                                              #
# Required global variables:                                                   #
#   @OBJECTS     : object files                                                #
#   $LDFLAGS     : link flags                                                  #
#   $LIBRARIES   : libraries list for linking                                  #
#   $TESTPROGRAM : executable name                                             #
#   $ARGUMENTS   : executable arguments                                        #
#   $EXECMODE    : execution mode ("NORMAL" or "DEBUG")                        #
#   $TDFFILE     : generated TDF file                                          #
#   $AMLFILE     : generated AML report                                        #
#   @OTSFILES    : list of OTS files                                           #
#   $BINDIR      : ATTOL binaries directory                                    #
#   $SCRIPTDIR   : Studio scripts directory                                    #
#   $TARGETDIR   : target package directory                                    #
#   $LOGFILENAME : generated files log                                         #
################################################################################

# To do output in chronological order when stderr and stdout are both used
use FileHandle;
STDOUT->autoflush(1);
STDERR->autoflush(1);

# Configuration file
require $ARGV[0] unless($main::TestRTCC);

# Miscellaneous functions 
require $SCRIPTDIR . "Fcts.pl";
require $SCRIPTDIR . "Tools.pl";
require $SCRIPTDIR . "Split.pl";

# Target package related functions
require $SCRIPTDIR . "Products.pl";
require $SCRIPTDIR . "CmsInfo.pl";

# Target-dependant functions
require $TARGETDIR . "/cmd/atl_func.pl";
require $TARGETDIR . "/cmd/atl_link.pl";
require $TARGETDIR . "/cmd/atl_exec.pl";
require $SCRIPTDIR . "codeChecker.pl" if($MISRA_MODE);

################################################################################
# Attol ObjecTest postprocessor execution
# Arguments:
#    $tdf    : traces file
#    $ots    : static correspondence files (space-separated list)
#    $xrd    : generated AML report
# Return value: atopostpro return value
# Required global variables:
#    $BINDIR
# Used global variables:
################################################################################
sub ato_postpro 
{
   my $tdf = shift;
   my $ots= shift;
   my $xrd= shift;

  my $headFile="$NAME.summary.txt";
  my $headOption;
  if(open (HDR,">$headFile")){
     if($RTRT_ACTIVATED){ #=1 on eclipse
	print HDR "Project=$PROJECT_NAME\n";
	print HDR "Project configuration=$CONFIGURATION\n";
	print HDR "Workspace=$WORKSPACE_PATH\n";
	print HDR "Test name=$NAME\n";
	print HDR "Report file=$xrd\n";
	print HDR "\$PREPROCESS\n";
     }else{
	print HDR "Project=$PROJECT_NAME\n";
	print HDR "Project file=$PROJECT_PATH.rtp\n";
	print HDR "Project configuration=$CONFIGURATION\n";
	print HDR "Workspace=$WORKSPACE_NAME\n";
	print HDR "Test node=$NAME\n";
	print HDR "Report file=".absolutePath($xrd)."\n";
	print HDR "\$PREPROCESS\n";
     }
#	print HDR "=$\n";
     if(($FILEVERCMS) && ($FILEVERCMS ne '0'))
     {
        print &PerlTranslate("UnitTesting.pl", "Check file's version\n", "");
        print HDR "#Tested files version\n";
 	foreach my $f (@SOURCES) #(@INTEGRATEDFILENAMES,@ADDITONALFILES)
 	{
 		my $v=getFileVersion($f);
		$f=absolutePath($f);
 		print HDR "$f=$v\n";
	}
     }
     close HDR;
     $headOption=" -h \"$headFile\"";
  }

  if($Ini{'REPORTHEADER'}) {
     print "REPORTHEADER specified as:$TARGETDIR/cmd/$Ini{'REPORTHEADER'}\n";
     print "sub reportHeader($headFile,$RTRT_ACTIVATED)\n";
     require "$TARGETDIR/cmd/$Ini{'REPORTHEADER'}";
     reportHeader($headFile,$RTRT_ACTIVATED);
  }

   # command
   my $Command = "$BINDIR" . "$PATH_SEPARATOR" . "atopospro -ots $ots -tdf \"$tdf\" -xrd \"$xrd\" $headOption";

   # execution
   my $Status = SystemP ($Command);

   Cat($headFile,"$NAME.atc.txt") if(($ATCMODE) && (-f "$NAME.atc.txt"));
   my $memSum=$ATP_TPF_NAME; $memSum=~s/\.tpf$/.atp.txt/;
   Cat($headFile,$memSum) if(($ATPMODE) && (-f $memSum));
   RegisterTmpFiles($headFile);

   if($ENV{RTRT_TEST_SUM}){
     Cat($ENV{RTRT_TEST_SUM},$headFile);
   }

   return $Status;
}

################################################################################
# wait for the end of the result file growing
sub wait_close  
{
      # Initialization
      my $count_lines=0;
      my $start_count_line=0;
      my $start_time = time();
      my $current_time = time();
      my $end=0;
      
      # While the file grows do nothing.
      # when the file does not grow anymore, run the atopospro
      while (!$end) {
	  if (open(INPUTFL, $TDFFILE))
	  {
	      $count_lines=0;
	      while(<INPUTFL>)
		{
		  $count_lines++;
		}

	      $current_time = time();

	      #if has grown.
	      if (($count_lines - $start_count_line) > 0)
		{
		  $start_count_line = $count_lines;
		  $count_lines=0;
		  $start_time = $current_time;
		} else {
		  #same file.
		  if (($current_time - $start_time) > $wait_time)
		    {
		      $end=1;
		    }
		}
		unless ($end)
		{
		  sleep($wait_time);
		}
	  } else {
	      #secure if there is no TDF file.
	      sleep($wait_time);
	      unless (open(INPUTFL, $TDFFILE))
	      {
		  $current_time = time();
		  if ($current_time - $start_time > $wait_time)
		  {
		      $end=1;
		  }
	      }
	  }
      }
}

################################################################################
# link, execute and postprocess for ATO node
sub ObjectTest
{
my $status = 0;
my $codeChecker_status="x";

$OUTFILE = "$ATL_OUT_FILE";

($tdfdir, $tdfname, $tdfext) = SplitFileName ($OUTFILE);

#--------------------------------------------------------------------------
## Misra
if($MISRA_MODE && @XOB) {
   if ($STAGE =~ /e/ ) {
      $codeChecker_status=codeChecker_link() if($STAGE =~ /l/ );
   }else{
      $codeChecker_status=codeChecker_link();
   }
}
#----------------------------------------------------------------------------
## Linkage
#----------------------------------------------------------------------------
if ($STAGE =~ /l/) 
{
  #----------------------------------------------------------------------------
  if ( $ATL_RUNTIME_FORMAT ne "NONE") {
    ## do not forget object file of target package.
    if (MakeTargetPackageObject()) {
      return 1;
    }
    if($ATL_RUNTIME_FORMAT eq "OBJ") {
       push( @OBJECTS, GetTargetPackageLib($OBJEXT) );
    } else {
       push( @ADDITIONAL_LIBRARIES, GetTargetPackageLib($LIBEXT) );
    }
  }
      
  print &PerlTranslate("ObjecTest.pl", "Linking %1...\n", "%1 is the program to be linked", $TESTPROGRAM);
  $status = atl_link ($TESTPROGRAM, \@OBJECTS, $LDFLAGS,
                      \@LIBRARY_DIR_LIST, $LIBRARIES, \@ADDITIONAL_LIBRARIES);
  EndStep($status);
  if ($status != 0 ) { die &PerlTranslate("ObjecTest.pl", "Failed.\n"); }
  
  ## Do not forget the OBJ file in output log
  open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("ObjecTest.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
  print  LOG "EXE $TESTPROGRAM\n";
  close( LOG );
  if ( $codeChecker_status eq ""){
    	print &PerlTranslate("ObjecTest.pl", "Code review report generation failed\n", "");
    	return 1;
  }
}

#----------------------------------------------------------------------------
## Execution
#----------------------------------------------------------------------------
if ($STAGE =~ /e/ )
{
  print &PerlTranslate("ObjecTest.pl", "Executing %1 %2...\n", "%1 is the program to be executed and %2 contains the arguments given to it", $TESTPROGRAM, $ARGUMENTS);
  $status = AtlExec ($TESTPROGRAM, $OUTFILE, $ARGUMENTS);
  EndStep($status);
  if ($status != 0 )
  {
    print &PerlTranslate("ObjecTest.pl", "Executable terminated with error code %1.\n", "%1 is the error code number", $status);
  }

    if (( ! -f $OUTFILE) && ( ! $SOCKET_UPLOAD ))
    {
      print &PerlTranslate("ObjecTest.pl", "No trace generated.\n");
      return(1);
    }

    SplitTraces ($OUTFILE);

    if ( ! $used[$tdf] )
    {
      print &PerlTranslate("ObjecTest.pl", "No TDF instruction found.\n");
      return(1);
    }
}

#----------------------------------------------------------------------------
## Postprocessing
#----------------------------------------------------------------------------
if ($STAGE =~ /r/ )
{
  # GD: wait for end of execution before running atopospro - tempo (cf DR 2021)
  my $wait_time = 2;
  if ($SOCKET_UPLOAD)
    {
      my $t=5;
      while($t && !(-f "trtpd.log")){
	sleep($wait_time);
	$t--;
      }	
    }
  # not used since studio generate endtest.txt and trtpd generate trtpd.log allowing to SplitTraces to wait for the end of the test. thanks to "tdp end" token generated by the runtime
  #wait_close() if (($ATTMODE && $ATT_ON_THE_FLY) || ($SOCKET_UPLOAD)); #still there for required special conf
  print &PerlTranslate("ObjecTest.pl", "Test Report Generation...\n");

  if ($ATCMODE && $ATC_SUMMARY){
    if ( ! $used[$tio] )
    {
      print &PerlTranslate("ObjecTest.pl", "No TIO instruction found.\n");
      $ATC_REPORT_FAILED=1;
    }
    else
    {
      $status = atc_postpro ($ATC_TIO_NAME, '');
    }
  }

  $otslist = "";
  foreach $ots ( @OTSFILES )
  {
     $otslist .= "\"$ots\" ";
  }
  $status = ato_postpro ($TDFFILE, $otslist, $XRDFILE);
  if ($status != 0 ) { print &PerlTranslate("ObjecTest.pl", "Failed.\n"); return(1); }
  open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("ObjecTest.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
  print  LOG "XRD $XRDFILE\n";
  close( LOG );
}
  return 0;
}

################################################################################
# Here is the beginning
################################################################################

unless($main::TestRTCC){

CheckIntDir();
CheckOutDir();
EnterInScript();

ClearLogFile($LOGFILENAME);
exit (ObjectTest());
}
1;
#
################################################################################
