/**********************************************************************
 *  Module  : rtrtWtx2.c
 *  Authors : R.France & B.Cormier & S.Washbrook (IBM Rational)
 *  Date    : 17 Jan 2005
 *
 *********************************************************************/

#include <stdlib.h>
#include <stdio.h>
#ifdef _WIN32
#include <windows.h>
#include <process.h>
typedef long thread_t;
#else
#include <pthread.h>
#include <thread.h>
#define LPTHREAD_START_ROUTINE void *(*)(void *)
#define CreateThread thr_create
#endif
#include <string.h>
#include <signal.h>
#include <errno.h>
#include <wtx.h>

/* Use the event notification instead of event polling */
#define EVENTHANDLER

/* Max number of modules to load on the target */
#define NB_MODULES_MAX 100

typedef struct file_s {
  int fNo;
  FILE *fH;
  struct file_s *next;
} file_t;

file_t *fh=NULL;

HWTX hWtx;
char DEBUG=0;
char DISPLAYEVT=0;
char NOSTOP=0;
char stopAtExit=0;
char load=0;
char Exec=0;
char unload=0;
char reset=0;
char traceActiv=1;
char listenActiv=0;
char certKernel=0;
enum {
	up=0,
	initialized,
	attached,
	loading,
	loaded,
	started,
	Tcollected,
	unloaded
} execState = up;
int sleepTime = 9;
char aborted = 0;
char entry[25];
char vxTgt[100];
int wtxtimeout = 0;
int currTimeout = 0;
int wtxtimeoutset = 0;

char fname[200]="atlout.spt";
char *mode="w";
int nbOpen=0;
char *objArray[NB_MODULES_MAX];
char objPathName[1024];

int nbModules;
int delayBeforeAttach=30;

/**********************************************************************
* Service : waitFor
* This function is waiting for x seconds (x given as a parameter)
**********************************************************************/
void waitFor(int s)
{
#ifdef _WIN32
	Sleep(s*1000);
#else
	sleep(s);
#endif
}

/**********************************************************************
* Service : blocking
* This function stops the execution until ctrl+C
**********************************************************************/
void blocking(int ex)
{
  aborted = 0;
  printf("ctrl+C to terminate\n");
  while(!aborted) waitFor(1);
  exit(ex);
}

/**********************************************************************
* Service : CloseAllFiles
* This function closes all traces file
**********************************************************************/
void CloseAllFiles()
{
file_t *newFH,*nextFH;
  newFH = fh;
  while(newFH)
  {
	fclose(newFH->fH);
	nextFH = newFH->next;
	free(newFH);
	newFH=nextFH;
  }
nbOpen=0;
DEBUG && printf ("All File closed\n");

}
/**********************************************************************
* Service : handleEvt
* This function is handling the events sent by the target. There
* is 4 kind of events : one to open a file, one to close a file,
* one to write a string in a file, and one to close all the open files.
* Events are :
* 	RTRTOPENF_<filenumber>_<filename>
* 	RTRTCLOSE_<filenumber>
* 	RTRTWRITE_<filenumber>_<buffer>
* 	RTRTMSG_<buffer>
*   	RTRTENDS
**********************************************************************/
void handleEvt(char *evt)
{
  char *tmp, fNumber[6];
  file_t *newFH, *prevFH;
  int i, fNo;
  DISPLAYEVT && printf("%s\n",evt);
  if (tmp = strstr(evt, "RTRTWRITE_"))
    {
      tmp = tmp + 10;
      for(i=0; i<5; i++)
		{
		  fNumber[i]=*tmp++;
		}
      fNumber[i]=0;
	  tmp++; /* skip 1 char */
      fNo = atoi(fNumber);
      newFH = fh;
      while((newFH)&&(newFH->fNo!=fNo))
		newFH = newFH->next;

      if (!newFH)
        {
          fNo=10000;
		  newFH = fh;
		  while((newFH)&&(newFH->fNo!=fNo))
			newFH = newFH->next;
			if(newFH == NULL)
			{
			  newFH = (file_t *)malloc(sizeof(file_t));
			  newFH->fNo = fNo;
			  newFH->fH = fopen(fname, mode);
			  newFH->next = fh;
			  fh = newFH;
			  printf("write without open, So Buffer written to %s\n",fname);
			}
        }
	  fputs(tmp,newFH->fH);
	  fflush(newFH->fH);
    }
  else if (tmp = strstr(evt, "RTRTOPENF_"))
    {
      tmp = tmp + 10;
      for(i=0; i<5; i++)
		{
		  fNumber[i]=*tmp++;
		}
      fNumber[i]=0;
      tmp++;

      newFH = (file_t *)malloc(sizeof(file_t));
      newFH->fNo = atoi(fNumber);
      newFH->fH = fopen(tmp, mode);
      newFH->next = fh;
      fh = newFH;
      nbOpen++;
      DEBUG && printf ("File opened fNumber = <%s>\n", fNumber);
    }
  else if (tmp = strstr(evt, "RTRTCLOSE_"))
    {
      tmp = tmp + 10;
      fNo = atoi(tmp);
      prevFH = NULL;
      newFH = fh;
      while((newFH)&&(newFH->fNo!=fNo))
      	{
			prevFH = newFH;
			newFH = newFH->next;
		}
      if(newFH)
      {
		fclose(newFH->fH);

		if(prevFH)
		{
      		prevFH->next = newFH->next;
		}
		else
		{
			fh=newFH->next;
		}
      	free(newFH);
      	nbOpen--;
      	DEBUG && printf ("File closed tmp = <%s>\n", tmp);
	  }
      if(nbOpen==0) traceActiv=0;
    }
  else if (tmp = strstr(evt, "RTRTENDS"))
    {
	/* close all files */
	CloseAllFiles();
	traceActiv=0;
    }
  else if (tmp = strstr(evt, "RTRTMSG_"))
    {
	tmp = tmp + 8;
	printf("\n%s\n", tmp);
    }
  else
    {
      printf("Unknown event : %s\n", evt);
    }
#ifndef EVENTHANDLER
 /* Reset the sleep time to zero so as not to miss an event */
 sleepTime = 0;
#endif
}

#ifdef EVENTHANDLER
/**********************************************************************
* Service : handleEvt
* This function is handling the events sent by the target. There
**********************************************************************/
void eventHandler (WTX_EVENT_DESC *evt)
{
  if (evt!=NULL)
    {
      if ((evt->event !=NULL) && (strlen(evt->event)>2))
	{
	  if (strstr(evt->event, "VIO_WRITE"))
	    {
	      /* ---------------------CERT-------------------*/
	      DEBUG && printf("%s (%d)>%s<\n",evt->event,evt->addlDataLen,evt->addlData);
	      if (evt->addlDataLen && strstr(evt->addlData, "RTRT"))
		{
		  handleEvt(evt->addlData);
		}
	      /* ---------------------CERT-------------------*/
	    }
	  else
	    {
	      if (strstr(evt->event, "RTRT"))
		{
		  handleEvt(evt->event);
		}
	    }
	}
    }
}
#endif
/**********************************************************************
* Service : terminateWtx
* This function is called before exiting this application.
**********************************************************************/
void terminateWtx(HWTX hWtx, int ex)
{
int j;
  switch(execState)
    {
    case loaded:
    case started:
    case Tcollected:
    case loading:
    /* unload in reverse order */
      if (unload)
	{
	  printf ("Unloading objects in reverse order\n");
	  j=nbModules;
	  while((j>=0)&&(objArray[j]!=NULL))
	    {
	      if (wtxObjModuleByNameUnload(hWtx, objArray[j])!=WTX_OK)
		{
		  printf("Unable to unload %s\n", objArray[j]);
		  ex=1;
		}
	      j--;
	    }
	}
    case attached:
#ifdef EVENTHANDLER
	  if(listenActiv)
	  {
	   wtxAsyncNotifyDisable(hWtx);
	   CloseAllFiles();
	  }
#endif
      if (reset)
      {
		  printf("resetting the target...\n");
		  wtxTargetReset(hWtx); /* on -reset option we reset the target before the end */
	  }
      wtxToolDetach(hWtx);
    case initialized:
      wtxTerminate(hWtx);
    case up:
    case unloaded:
      if(stopAtExit) blocking(ex);
      else if((DEBUG || ex) && !NOSTOP) blocking(ex);
      break;
    default:
      printf("Unknown State");
    }
  exit(ex);
}

#ifndef EVENTHANDLER
/**********************************************************************
* Service : getEvt
* This function is waiting for events in a loop. It exits when all
* the files have been closed.
**********************************************************************/
void * getEvt(void * no)
{
  WTX_EVENT_DESC *evt;

  DEBUG && printf("Waiting for event ...\n");
  while (traceActiv && (aborted == 0))
  {
	evt = wtxEventGet(hWtx);
	if ((evt!=NULL) && (strlen(evt->event)>2))
	{
		  handleEvt(evt->event);
	}
	wtxResultFree(hWtx, evt);

	/* sleepTime is reset to zero every time a TestRT event is processed */
	if (sleepTime > 0)
		waitFor(sleepTime);

	if (sleepTime < 8)
		sleepTime ++;
  }
  CloseAllFiles(); /* when aborted files are not closed yet */
  traceActiv=0;
  /* all files are close: now try to unload objects in reverse order */
  /* die */
  return &traceActiv; /* return anything to be void * compliant */
}

void SpawnGetEvt()
{
  thread_t ThreadId;

  CreateThread(
	NULL,  /*  pointer to security attributes */
	0,     /*  initial thread stack size */
	(LPTHREAD_START_ROUTINE) getEvt,/*  pointer to thread function */
	(void *)&hWtx,     /*  argument for new thread */
	0,     /*  creation flags */
	&ThreadId);/*  pointer to receive thread ID); */

  waitFor(1);
}
#endif
/***************************************************************************
 * Trap the control-C to exit cleanly
 ***************************************************************************/
void Control_C_Handler (int sig)
{
  aborted = 1;
  printf("signal %d...RTRT WTX shuting down!!!...\n",sig);
}

/***************************************************************************
 * Get the object files list
 ***************************************************************************/
void buildObjectList(int i,char *name)
{
  FILE* pFile;
  objArray[i]= name;
  DEBUG && printf ("\tobject[%d] = <%s>\n", i, objArray[i]);
  /* Check that the file exists */
  pFile = fopen (objArray[i], "r");
  if (pFile == 0)
  {
	printf ("ERROR: The file <%s> does not exist. Check the path to this file\n", objArray[i]);
	terminateWtx(hWtx, 1);
  }
  fclose(pFile);
}

/***************************************************************************
 * Main routine
 **************************************************************************/

void main(int argc, char *argv[])
{
  STATUS status;
  WTX_SYMBOL *wtxSym, *tmpSym;
  WTX_LD_M_FILE_DESC *pFileDesc, *pFileDescTmp;
  WTX_CONTEXT_DESC *pContextDesc;
  char *tmp,*tmps;
  char tmp2[1024];
  int i,j,k,tryNB;
  WTX_SYM_LIST *lwtxSym;

  signal (SIGINT, Control_C_Handler);
  signal (SIGTERM, Control_C_Handler);
  signal (SIGABRT, Control_C_Handler);
  signal (SIGILL, Control_C_Handler);
  signal (SIGFPE, Control_C_Handler);
  signal (SIGSEGV, Control_C_Handler);
#ifdef _WIN32
  signal (SIGBREAK, Control_C_Handler);
#endif

  objPathName[0] = '\0';
  vxTgt[0] = '\0';
  for (i=1; i<argc; i++)
    {
	  DEBUG && printf ("Treating argument %d <%s>\n", i, argv[i]);
	  if (strcmp(argv[i], "-entry") == 0)
		{
		  i++;
		  if ((i >= argc) || (argv[i][0] == '-'))
			{
			  printf ("Missing argument for -entry\n");
			  terminateWtx(hWtx, 1);
			}
		  Exec=1;
		  strcpy(entry, argv[i]);
		}
	  else if (strcmp(argv[i], "-list") == 0)
		{
		  i++;
		  if ((i >= argc) || (argv[i][0] == '-'))
			{
			  printf ("Missing argument for -list\n");
			  terminateWtx(hWtx, 1);
			}
		  strncpy(objPathName, argv[i],1024);
		}
	  else if (strcmp(argv[i], "-load") == 0)
		{
		  load=1;
		}
	  else if (strcmp(argv[i], "-unload") == 0)
		{
		  unload=1;
		}
	  else if (strcmp(argv[i], "-reset") == 0)
		{
		  reset=1;
		}
	  else if (strcmp(argv[i], "-notraces") == 0)
		{
			traceActiv=0;
		}
	  else if (strcmp(argv[i], "-a") == 0)
		{
			mode="a";
		}
	  else if (strcmp(argv[i], "-out") == 0)
		{
		  i++;
		  if ((i >= argc) || (argv[i][0] == '-'))
			{
			  printf ("Missing argument for -out\n");
			  terminateWtx(hWtx, 1);
			}
 	      DEBUG && printf ("\tFileName is <%s>\n", argv[i]);
		  strcpy(fname, argv[i]);
		}
	  else if (strcmp(argv[i], "-wtxtimeout") == 0)
		{
		  i++;
		  if ((i >= argc) || (argv[i][0] == '-'))
			{
			  printf ("Missing argument for -wtxtimeout\n");
			  terminateWtx(hWtx, 1);
			}
		  wtxtimeoutset = 1;
		  wtxtimeout = atoi(argv[i]);
		  if (wtxtimeout == 0) {
		    printf ("Argument to -wtxtimeout \"%s\" is not an integer, or is zero\n", argv[i]);
		    terminateWtx(hWtx, 1);
		  }
 	      DEBUG && printf ("\t-wtxtimeout is <%s>\n", argv[i]);
		}
	  else if (strcmp(argv[i], "-t") == 0)
		{
		  i++;
		  if ((i >= argc) || (argv[i][0] == '-'))
			{
			  printf ("Missing argument for -t\n");
			  terminateWtx(hWtx, 1);
			}
 	      DEBUG && printf ("\ttargetName is <%s>\n", argv[i]);
		  strcpy(vxTgt, argv[i]);
		}
	  else if (strcmp(argv[i], "-cert") == 0)
		{
		  certKernel=1;
		}
	  else if (strcmp(argv[i], "-nostop") == 0)
		{
		  NOSTOP=1;
		}
	  else if (strcmp(argv[i], "-v") == 0)
		{
		  DEBUG=1;
		}
	  else if (strcmp(argv[i], "-dump") == 0)
		{
		  DISPLAYEVT=1;
		}
	  else if (strcmp(argv[i], "-b") == 0)
		{
		  stopAtExit=1;
		}
	  else if (strcmp(argv[i], "-delay") == 0)
		{
		  i++;
		  if ((i >= argc) || (argv[i][0] == '-'))
			{
			  printf ("Missing argument for -delay\n");
			  terminateWtx(hWtx, 1);
			}
		  delayBeforeAttach=atoi(argv[i]);
		  if (delayBeforeAttach <= 0)
			{
			  printf ("Argument for -delay is not an integer greater than 0.\n");
			  terminateWtx(hWtx, 1);
			}
		}
	  else
		{
		  printf ("Unknown option <%s>\n",argv[i]);
		  printf ("Usage:\n\trtrtwtx [options]\nWhere options can be one or more of the following:\n");
		  printf ("\t-entry <entry point> : Name of the function to start the application.\n");
		  printf ("\t-list <comma separated list of objects to load/unload> or .lst file\n");
		  printf ("\t-load : Load then execute the objects listed in -list option\n");
		  printf ("\t-unload : Unload after execution, the objects listed in -list option\n");
		  printf ("\t-reset : reset the target after execution and unload (if any)\n");
		  printf ("\t-cert : Using the CERT Kernel\n");
		  printf ("\t-dump : display any received event from the target\n");
		  printf ("\t-nostop : let exit what ever the events\n");
		  printf ("\t-out <default out File Name> : Name of the default out spt file written\n");
		  printf ("\t-notraces : Do not get result file trace from target\n");
		  printf ("\t-wtxtimeout : Specifies the time in miliseconds before calls to the WTX API time out. If you get API_REQUEST_TIMED_OUT error messages you need to increase this value.\n");
		  printf ("\t-t <Target Name> : Name of the Tornado target to attach to\n");
		  printf ("\t-a : Files opened in append mode\n");
		  printf ("\t-v : Verbose mode\n");
		  printf ("\t-b : Stops when the program ends (do not exits)\n");
		  printf ("\t-delay <delay> : Try every second for <delay> seconds to attach to the target server\n");
		  printf ("\n");
		  terminateWtx(hWtx, 1);
		}
    }
  if(Exec && !load)
  {
	  printf ("ERROR: -entry cannot be used without -load\n");
	  terminateWtx(hWtx, 1);
  }
  if(Exec && unload && !traceActiv)
  {
	  printf ("ERROR: -load and -unload cannot be used together when -notraces is selected\n");
	  terminateWtx(hWtx, 1);
  }
  if(vxTgt[0]==0)
  {
	  printf ("ERROR: -t <Target Name> is a mandatory option\n");
	  terminateWtx(hWtx, 1);
  }
  if((objPathName[0]==0) && (load || unload))
  {
	  printf ("ERROR: -load and -unload options must be used in conjunction of -list\n");
	  terminateWtx(hWtx, 1);
  }

  /* Modules parsing */
  for (i=0; i<NB_MODULES_MAX; i++)
    objArray[i]=NULL;

  if (objPathName[0] != 0)
  {
    DEBUG && printf ("Objects list = <%s>\n", objPathName);
    for(tmps=tmp=objPathName, k=1, j=0, i=0; k; tmp++)
    {
      if(*tmp=='.') j=1;
      else if((j==1) && (*tmp=='l')) j=2;
      else if((j==2) && (*tmp=='s')) j=3;
      else if((j==3) && (*tmp=='t')) j=4;
      else if ((*tmp==',')||(*tmp==0))
		{
		  if(*tmp==0) k=0;
		  *tmp=0; /* mark end of string */
		  if(j==4) /* means .lst file */
		  {
			  FILE* pFileLst;
		  	  DEBUG &&  printf ("list of file %s\n",tmps);
			  pFileLst = fopen (tmps, "r");
			  if (pFileLst == 0)
			  {
				  printf ("ERROR: The file <%s> does not exist. Check the path to this file\n", tmps);
				  terminateWtx(hWtx, 1);
			  }
			  else
			  {
			    while(fgets(tmp2,1024,pFileLst) != 0)
				  {
					  j=strlen(tmp2);
					  tmps=(char *)malloc(j+1);
					  strcpy(tmps, tmp2);
					  while((tmps[j] == 0) || (tmps[j]=='\n') || (tmps[j]=='\r'))
					  {
						  tmps[j--]=0;
					  }
					  buildObjectList(i++,tmps);
				  }
				  fclose(pFileLst);
			  }
		  }
		  else
		  {
			  buildObjectList(i++,tmps);
	  	  }
		  tmps=tmp;
		  tmps++; /* start of name on the next name */
		  j=0;
		}
      if (i > NB_MODULES_MAX)
		{
		  printf ("ERROR: This tool is limited to %d modules, you have exceeded this limit!\n", NB_MODULES_MAX);
		  terminateWtx(hWtx, 1);
		}
    }
  }

  /*****************************************
  * Initializing the WTX layer *
  ****************************************/
  if (wtxInitialize(&hWtx)!=WTX_OK)
    {
	printf("Init Error ...\n");
	terminateWtx(hWtx, 1);
    }
  DEBUG && printf ("wtxInitialize OK\n");
  execState=initialized;

  /*****************************************
  * Attaching to the target server *
  ****************************************/
  tryNB = 0;
  while ((status=wtxToolAttach(hWtx, vxTgt, "wtxapp")!=WTX_OK) && (tryNB < delayBeforeAttach))
    {
	tryNB++;
	if (tryNB == 1)
	{
		printf ("Unable to attach to the target server <%s>\n", vxTgt);
		printf ("Will retry every second for %d seconds\n", delayBeforeAttach);
	}
  	DEBUG && printf ("Trying to attach (%d/%d)\n", tryNB, delayBeforeAttach);
	waitFor(1);
    }
  if (tryNB == delayBeforeAttach)
    {
	printf("Attach Error ... \n");
	terminateWtx(hWtx, 1);
    }
  DEBUG && printf ("wtxToolAttach OK\n");
  execState=attached;

  /*****************************************
  * Setting the WTX timeout
  ****************************************/
  wtxTimeoutGet(hWtx, &currTimeout);
  DEBUG && printf ("current WTX timeout = %d milliseconds\n", currTimeout);
  if (wtxtimeoutset) {
    if (wtxTimeoutSet(hWtx, wtxtimeout) != WTX_OK) {
      WTX_ERROR_T errCode = wtxErrGet(hWtx);
      const char* pMsgErr = wtxErrToMsg (hWtx, errCode);
      printf ("Error setting the time out to %d milliseconds because\n%s\n", wtxtimeout, pMsgErr);
      terminateWtx(hWtx, 1);
    }
  }

  if (traceActiv)
    {
	  /*****************************************
			Getting traces
	  ****************************************/
	  /* Registering to user events*/
      if (certKernel)
	{
	  /*---------------- CERT -----------------------*/
 	  if (wtxRegisterForEvent(hWtx, "[UV][SI].*")!=WTX_OK)
	    {
	      printf("Unable to register for VIO event\n");
	      terminateWtx(hWtx, 1);
	    }
	  /*---------------- CERT -----------------------*/
	}
      else
	{
	  if (wtxRegisterForEvent(hWtx, "USER.*")!=WTX_OK)
	    {
	      printf("Unable to register for event\n");
	      terminateWtx(hWtx, 1);
	    }
	}
	  /* start thread for Event */
#ifdef EVENTHANDLER
	  DEBUG && printf("Waiting for event ...\n");
	  if(wtxAsyncNotifyEnable(hWtx,(FUNCPTR)eventHandler) != WTX_OK)
	    {
	  	  printf("Unable to Notify for event\n");
	  	  terminateWtx(hWtx, 1);
	    }
	  listenActiv=1;
#else
	  SpawnGetEvt();
#endif
	}
  if (load)
    {
      /*****************************************
      * Loading object files to target *
      ****************************************/
      execState=loading;
      printf ("Loading objects onto target\n");

      pFileDesc=(WTX_LD_M_FILE_DESC *)malloc(sizeof(WTX_LD_M_FILE_DESC)+1024);
      if (pFileDesc == 0)
		{
		  DEBUG && printf ("Loading 1: malloc of %d bytes failed\n",sizeof(WTX_LD_M_FILE_DESC)+1024);
		  printf("Unable to load...\n");
		  terminateWtx(hWtx, 1);
		}

      j=0;
      while((objArray[j]!=NULL)&&(j<=NB_MODULES_MAX)&&(!aborted))
		{
		  int Loaded = 0;
		  DEBUG && printf ("Loading the module <%s>...", objArray[j]);
		  pFileDesc->filename=objArray[j];
		  pFileDesc->loadFlag=0x6;
		  pFileDesc->nSections=0;
		  pFileDesc->moduleId=WTX_LOAD_FROM_CLIENT;
		  nbModules=j;
		  i = 5;
		  while ((!Loaded) && (!aborted) && (i > 0))
			{
			  pFileDescTmp=wtxObjModuleLoad(hWtx, pFileDesc);
			  if(pFileDescTmp == NULL)
				{
				  i--; /* Retry */
				  waitFor (1);
				  DEBUG && printf ("retry...");
				}
			  else
				Loaded = 1;
			}
		  if (Loaded)
			{
			  DEBUG && printf ("Loaded\n");
			}
		  else
			{
			  /* Get the reason why the load failed */
			  WTX_ERROR_T errCode = wtxErrGet(hWtx);
			  const char* pMsgErr = wtxErrToMsg (hWtx, errCode);
			  printf ("\nUnable to load <%s> because\n\t%s\n",
					   objArray[j],
					   pMsgErr);
			  terminateWtx(hWtx, 1);
			}

		  /* Checking for unresolved symbols */
		  if ((tmpSym=pFileDescTmp->undefSymList.pSymbol)!=NULL)
			{
			  do
				{
				  printf("Unresolved symbol : %s (%s)\n", tmpSym->name, objArray[j]);
				  tmpSym = tmpSym->next;
				}
			  while ((tmpSym) && (!aborted));
			}
		  wtxResultFree(hWtx, pFileDescTmp);
		  j++;
		}
	  if(aborted) terminateWtx(hWtx, 1);
	  DEBUG && printf ("All listed modules loaded!!!\n");
	  execState=loaded;
	}
  if (Exec)
    {

	  /*****************************************
			Execution part
	  ****************************************/
	  printf("Start Execution from %s\n",entry);
	  /* Looking for entry-point */
	  tmp = (char *)malloc(256);
	  lwtxSym = wtxSymListGet(hWtx, entry, 0, 0, 0);
	  i=1;
	  while ((lwtxSym->pSymbol)&&(i))
		{
		  tmp = strstr(lwtxSym->pSymbol->name, entry);
		  tmp=tmp+strlen(entry);
		  if((*tmp==0)||((*tmp=='_')&&(*(tmp+1)=='_')))
			i=0;
		  if (i!=0)
			lwtxSym->pSymbol = lwtxSym->pSymbol->next;
		}
	  if (!(lwtxSym->pSymbol))
		  {
			printf("Unable to find entry point\n");
			terminateWtx(hWtx, 1);
		  }

	  if((wtxSym = wtxSymFind(hWtx, lwtxSym->pSymbol->name, 0, TRUE, 0, 0))==NULL)
		  {
			printf("wtxSymFind error\n");
			terminateWtx(hWtx, 1);
		  }
	  /* Spawning task on target */
	  pContextDesc=(WTX_CONTEXT_DESC *)malloc(sizeof(WTX_CONTEXT_DESC)+1024);
	  pContextDesc->contextType =4;
	  pContextDesc->returnType = 0;
	  pContextDesc->name = entry;
	  pContextDesc->priority = 100;
	  pContextDesc->options = 0;
	  pContextDesc->stackSize = 16000;
	  pContextDesc->entry = wtxSym->value;
      pContextDesc->redirIn = 0;
      pContextDesc->redirOut = 1;
      for(i=0; i<10; i++)
		pContextDesc->args[i] = 0;
      wtxFuncCall(hWtx, pContextDesc);
      wtxResultFree(hWtx, wtxSym);
      wtxResultFree(hWtx, tmpSym);
      DEBUG && printf("Execution started\n");
      execState=started;
    }
    /*****************************************
			wait end of execution
    ****************************************/
    /* wait for unload */
    while(traceActiv) waitFor(5);
    execState=Tcollected;

    /*****************************************
			Unload part
    ****************************************/
    /* all files are close: now try to unload objects in reverse order */
    if(unload==0) execState=attached;
    terminateWtx(hWtx, 0);
}

